/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000, 2001
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/AudioMaster.java,v 1.2 2003/05/07 22:47:24 ellis Exp $
 */

package xcard;

import java.net.URL;
import java.io.IOException;
import java.util.HashMap;
import javax.sound.midi.*;
import javax.sound.sampled.*;

// Must be able to use multiple clips by name


public class AudioMaster extends XCardBase {

    private double volume = 0.5;
    private boolean enabled = true;
    private HashMap map = new HashMap();
    private SoundThread soundThread = null;

    public AudioMaster()
    {
	soundThread = new SoundThread();
	soundThread.start();
    }

    public void setClip(String key, URL url)
	throws UnsupportedAudioFileException,
	       LineUnavailableException,
	       IOException
    {
	Clip clip = getClip(url);
	map.put(key, clip);
    }

    public void playClip(String key)
    {
	Clip clip = (Clip) map.get(key);
	if (clip != null) {
	    while (! soundThread.isReady()) {
		Thread.yield();
		try {
		    Thread.sleep(99);
		} catch(Exception e) {
		    e.printStackTrace();
		}
	    }
	    soundThread.play(clip);
	}
    }

    public boolean isClipCached(String key)
    {
	boolean exists = false;
	Object o = map.get(key);
	if (o != null) {
	    exists = true;
	}
	return exists;
    }

    private Clip getClip(URL url) throws UnsupportedAudioFileException,
					 LineUnavailableException,
					 IOException
    {
	Clip clip = null;
	AudioInputStream stream = AudioSystem.getAudioInputStream(url);
	AudioFormat format = stream.getFormat();
	DataLine.Info info = new DataLine.Info(Clip.class, format);
	clip = (Clip) AudioSystem.getLine(info);
	// May need to close it afterwards
	clip.open(stream);

	return clip;
    }


    private void playClip(Clip clip) throws LineUnavailableException
    {
	if (clip != null && enabled) {
	    // Volume control
	    FloatControl gainControl = (FloatControl) 
		clip.getControl(FloatControl.Type.MASTER_GAIN);
	    float db = (float) (Math.log(volume)/Math.log(10.0)* 20.0);
	    gainControl.setValue(db);
	
	    clip.start();
	    while (clip.isActive()) {
		Thread.yield();
		try {
		    Thread.sleep(99);
		} catch(InterruptedException ignore) {
		    ignore.printStackTrace();
		}
	    }
	    clip.stop();
	}
    }

    public void setVolume(int v)
    {
	volume = v / 10.0;
	if (volume == 0) {
	    disableSound();
	} else {
	    enableSound();
	}
    }

    public void enableSound()
    {
	enabled = true;
    }

    public void disableSound()
    {
	enabled = false;
    }

    public boolean isEnabled()
    {
	return enabled;
    }

    private class SoundThread extends Thread {
	private boolean ready = false;
	private Clip playClip = null;

	public void run()
	{
	    waitForRequest();
	}

	public synchronized boolean isReady()
	{
	    return ready;
	}

	public synchronized void setReady(boolean value)
	{
	    ready = value;
	}

	public synchronized void play(Clip clip)
	{
	    playClip = clip;
	    notify();
	}

	private synchronized void waitForRequest()
	{
	    while(true) {
		try {
		    setReady(true);
		    wait();
		    setReady(false);
		    playClip(playClip);
		} catch (InterruptedException e) {
		    e.printStackTrace();
		    break;
		} catch(LineUnavailableException e) {
		    e.printStackTrace();
		    break;
		    // ignore
		}
	    }
	}
    }



    public static void main(String[] argv)
    {
	try {
	    AudioMaster am =  new AudioMaster();
	    Resource resource = new Resource();
	    URL correctURL = resource.get("xcard/resources/corkpop2.wav");
	    URL incorrectURL = resource.get("xcard/resources/guitarboing.wav");

	    am.setClip("CORRECT", correctURL);
	    am.setClip("INCORRECT", incorrectURL);
	    
	    for (int i = 0; i < 5; i++) {
		am.playClip("CORRECT");
		am.playClip("INCORRECT");
	    }
	    System.exit(0);
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

}
