/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000, 2001
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/PreferencesService.java,v 1.7 2003/05/07 22:47:24 ellis Exp $
 */

package xcard;

import java.io.*;
import java.net.URL;
import java.util.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.*;

public class PreferencesService extends Service {

    private boolean soundEnabled = true;
    private int volume = 3;
    private String defaultFileDirectory = null;
    private Locale locale = Locale.getDefault();
    private String lookAndFeel = "Default";
    private String transDirection = "fr";

    static ResourceBundle resources;
    static {
	try {
	    resources = ResourceBundle.getBundle("xcard.XCardResourceBundle", 
						 Locale.getDefault());
	} catch (MissingResourceException mre) {
	    Message.errorMessage(mre);
	}
    }

    public PreferencesService() throws IOException
    {
	String home = System.getProperty("user.home");
	defaultFileDirectory = home + File.separatorChar + "xcard";

	readFile();
    }

    private void readFile() throws IOException
    {
	try {
	    String home = System.getProperty("user.home");
	    File file = new File(home + File.separatorChar + ".xcard");
	
	    if (file.exists()) {
		DocumentBuilderFactory docBuilderFactory = 
		    DocumentBuilderFactory.newInstance();
		docBuilderFactory.setValidating(false);
		DocumentBuilder docBuilder = 
		    docBuilderFactory.newDocumentBuilder();

		Document doc = docBuilder.parse(file);
		NodeList nlist = doc.getElementsByTagName("property");
		for (int index = 0; index < nlist.getLength(); index++) {
		    Node node = nlist.item(index);
		    assignValue(node);
		}
	    }

	} catch (Exception e) {
	    Message.errorMessage(e);
	}
    }

    private void assignValue(Node node)
    {
	String parameter = node.getNodeName();
	NamedNodeMap attrlist = node.getAttributes();
	    
	// Now process each parameter
	NodeList values = node.getChildNodes();
	for (int vindex = 0; vindex < values.getLength(); vindex++) {
	    Node valueNode = values.item(vindex);
	    String value = valueNode.getNodeValue();
	    assignValue(parameter, value, attrlist);
	}
    }
    
    private void assignValue(String parameter, String value, 
			     NamedNodeMap attrlist)
    {
	if (parameter.equals("property")) {
	    Node nameNode = attrlist.getNamedItem("name");
	    if(nameNode != null) {
		String name = nameNode.getNodeValue();
		if (name != null) {
		    if (name.equals("sound-enabled")) {
			if (value.equals("true")) {
			    soundEnabled = true;
			} else {
			    soundEnabled = false;
			}
		    } else if (name.equals("file-default-directory")) {
			defaultFileDirectory = value;
		    } else if (name.equals("locale")) {
			if (value.length() >= 5) {
			    locale = new Locale(value.substring(0, 2),
						value.substring(3, 5));
			    setLocale(locale);
			}
		    } else if (name.equals("sound-volume")) {
			volume = Integer.parseInt(value);
		    } else if (name.equals("look-and-feel")) {
			lookAndFeel = value;
		    } else if (name.equals("translate-to")) {
			transDirection = value;
		    }
		}
	    }
	}
    }

    private void writeProperty(OutputStreamWriter os, String prop,
			       String value)
	throws IOException
    {
	os.write("<property name=\"" + prop + "\">");
	os.write(value);
	os.write("</property>\n");

    }

    private void writeProperties(OutputStreamWriter os)
	throws IOException
    {
	writeProperty(os, "sound-volume", String.valueOf(volume));
	
	writeProperty(os, "file-default-directory",
		      defaultFileDirectory);
	writeProperty(os, "locale", locale.toString());
	writeProperty(os, "look-and-feel", lookAndFeel);
	writeProperty(os, "translate-to", transDirection);
    }

    public void commitChanges() throws IOException
    {
	OutputStreamWriter os = null;
	try {
	    String home = System.getProperty("user.home");
	    File file = new File(home + File.separatorChar + ".xcard");
	    os = new FileWriter(file);
	    
	    os.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
	    os.write("<settings>\n");
	    writeProperties(os);
	    os.write("</settings>\n");
	} finally {
	    if (os != null) {
		os.close();
	    }
	}
     }

    public boolean isSoundEnabled()
    {
	boolean value = false;
	if (volume > 0) {
	    value = true;
	}
	return value;
    }

    public int getVolume()
    {
	return volume;
    }

    public void setVolume(int volume)
    {
	this.volume = volume;
    }

    public String getDefaultDirectory()
    {
	return defaultFileDirectory;
    }

    public File getDefaultFileDirectory()
    {
	return new File(defaultFileDirectory);
    }

    public void setDefaultFileDirectory(String dir)
    {
	defaultFileDirectory = dir;
    }

    public void setLocale(Locale locale)
    {
	this.locale = locale;
	resources = ResourceBundle.getBundle("xcard.XCardResourceBundle", 
					     locale);
    }

    public ResourceBundle getResourceBundle()
    {
	return resources;
    }

    public boolean isLangFrench()
    {
	boolean result = false;
	String lang = locale.getLanguage();
	if (lang.equals("fr")) {
	    result = true;
	}
	return result;
    }

    public void setLookAndFeel(String val)
    {
	lookAndFeel = val;
    }

    public String getLookAndFeel()
    {
	return lookAndFeel;
    }

    public void setTranslationDirection(String value)
    {
	if (value != null && (value.equals("fr") || value.equals("en"))) {
	    transDirection = value;
	} else {
	    throw new IllegalArgumentException("Only accepts fr or en");
	}
    }

    public String getTranslationDirection()
    {
	return transDirection;
    }
    
}
